#include "debug_file.h"
#include "register.h"
#include "register_ioctl.h"

static void to_bitstring(uint64_t val, char* out) {
	out[64] = 0;

	for(unsigned int i = 0; i < 64; i++) {
		out[63 - i] = '0' + ((val >> i) & 1);
	}
}

#define SPRINTF_BUF_SIZE 768
static ssize_t read_impl(struct file *f, char __user *ubuf, size_t buf_size, loff_t *u_pos) {
	char temp_buf[SPRINTF_BUF_SIZE];
	int temp_len;
	int total_len = 0;

	if(f->f_pos > 0) {
		return 0;
	}

	temp_len = sprintf_cr0(get_cr0(), temp_buf, SPRINTF_BUF_SIZE);
	if(copy_to_user(ubuf + total_len, temp_buf, temp_len) > 0) {
		return -1;
	}
	total_len += temp_len;

	temp_len = snprintf(temp_buf, SPRINTF_BUF_SIZE, "CR2: 0x%p", (void *)get_cr2());
	temp_buf[temp_len++] = '\n';
	temp_buf[temp_len++] = 0;
	if(copy_to_user(ubuf + total_len, temp_buf, temp_len) > 0) {
		return -1;
	}
	total_len += temp_len;

	temp_len = sprintf_cr3(get_cr3(), temp_buf, SPRINTF_BUF_SIZE);
	if(copy_to_user(ubuf + total_len, temp_buf, temp_len) > 0) {
		return -1;
	}
	total_len += temp_len;

	temp_len = sprintf_cr4(get_cr4(), temp_buf, SPRINTF_BUF_SIZE);
	if(copy_to_user(ubuf + total_len, temp_buf, temp_len) > 0) {
		return -1;
	}
	total_len += temp_len;

	temp_len = snprintf(temp_buf, SPRINTF_BUF_SIZE, "CR8: ");
	to_bitstring(get_cr8(), temp_buf + temp_len);
	temp_len = strnlen(temp_buf, SPRINTF_BUF_SIZE);
	temp_buf[temp_len++] = '\n';
	temp_buf[temp_len++] = 0;
	if(copy_to_user(ubuf + total_len, temp_buf, temp_len) > 0) {
		return -1;
	}
	total_len += temp_len;

	*u_pos += total_len;

	return total_len;
}

static long ioctl_impl(struct file *f, unsigned int cmd, unsigned long arg) {
	enum command command = (enum command)cmd;

	switch(command) {
		case COMMAND_WP:
			set_wp(arg);
			break;
		case COMMAND_SMAP:
			set_smap(arg);
			break;
		case COMMAND_SMEP:
			set_smep(arg);
			break;
		default:
			return -ENOTTY;
	}
	
	return 0;
}

struct file_operations debug_fops = {
	.read = read_impl,
	.unlocked_ioctl = ioctl_impl
};
