#ifndef register_h_INCLUDED
	#define register_h_INCLUDED

#include <linux/module.h>

struct cr0_state {
	bool protection_enable;
	bool monitor_coprocessor;
	bool emulation;
	bool task_switched;
	bool extension_type;
	bool numeric_error;
	bool write_protect;
	bool alignment_mask;
	bool no_write_through;
	bool cache_disable;
	bool paging;
};

struct cr0_state get_cr0(void);
int sprintf_cr0(struct cr0_state s, char *buf, ssize_t n);

uint64_t get_cr1(void);
uint64_t get_cr2(void);

struct cr3_state {
	bool page_level_write_through;
	bool page_level_cache_disable;
	void *page_directory_base;
};

struct cr3_state get_cr3(void);
void set_cr3(uint64_t val);
int sprintf_cr3(struct cr3_state s, char *buf, ssize_t n);

struct cr4_state {
	bool virtual_mode_extensions;
	bool protected_mode_virtual_interrupts;
	bool time_stamp_disable;
	bool debugging_extensions;
	bool page_size_extensions;
	bool physical_address_extension;
	bool machine_check_enable;
	bool page_global_enable;
	bool perf_mon_counter_enable;
	bool os_unmasked_simd_float_exceptions;
	bool user_mode_instruction_prevention;
	bool linear_addres_57_bit;
	bool vmx_enable;
	bool smx_enable;
	bool fsgbase_enable;
	bool pcid_enable;
	bool extended_states_enable;
	bool key_locker_enable;
	bool smep_enable;
	bool smap_enable;
	bool user_mode_page_protection_keys;
	bool control_flow_enforcement_enable;
	bool supervisor_page_protection_keys;
};

struct cr4_state get_cr4(void);
int sprintf_cr4(struct cr4_state s, char *buf, ssize_t n);

uint64_t get_cr8(void);

struct efer_state {
	bool syscall_enable;
	bool linear_mode_enable;
	bool linear_mode_active;
	bool execute_disable;
};

struct efer_state get_efer(void);
int sprintf_efer(struct efer_state s, char *buf, ssize_t n);

void set_wp(bool state);
void set_smap(bool state);
void set_smep(bool state);

#endif
